@@ -1,3 +1,9 @@
+/*
+ * cog-platform-x11.c
+ * Copyright (C) 2020-2021 Igalia S.L.
+ *
+ * Distributed under terms of the MIT license.
+ */
 
 #include "../../core/cog.h"
 
@@ -44,12 +50,22 @@ typedef void (GL_APIENTRYP PFNGLEGLIMAGETARGETRENDERBUFFERSTORAGEOESPROC) (GLenu
 #define DEFAULT_WIDTH  1024
 #define DEFAULT_HEIGHT  768
 
-#if defined(WPE_CHECK_VERSION)
-# define HAVE_2D_AXIS_EVENT WPE_CHECK_VERSION(1, 5, 0) && WEBKIT_CHECK_VERSION(2, 27, 4)
-#else
-# define HAVE_2D_AXIS_EVENT 0
-#endif /* WPE_CHECK_VERSION */
+struct _CogX11PlatformClass {
+    CogPlatformClass parent_class;
+};
 
+struct _CogX11Platform {
+    CogPlatform parent;
+};
+
+G_DECLARE_FINAL_TYPE(CogX11Platform, cog_x11_platform, COG, X11_PLATFORM, CogPlatform)
+
+G_DEFINE_DYNAMIC_TYPE_EXTENDED(
+    CogX11Platform,
+    cog_x11_platform,
+    COG_TYPE_PLATFORM,
+    0,
+    g_io_extension_point_implement(COG_MODULES_PLATFORM_EXTENSION_POINT, g_define_type_id, "x11", 300);)
 
 struct CogX11Display {
     Display *display;
@@ -249,7 +265,6 @@ xcb_handle_key_release (xcb_key_press_event_t *event)
 static void
 xcb_handle_axis (xcb_button_press_event_t *event, const int16_t axis_delta[2])
 {
-#if HAVE_2D_AXIS_EVENT
     struct wpe_input_axis_2d_event input_event = {
         .base = {
             .type = wpe_input_axis_event_type_mask_2d | wpe_input_axis_event_type_motion_smooth,
@@ -262,26 +277,6 @@ xcb_handle_axis (xcb_button_press_event_t *event, const int16_t axis_delta[2])
     };
 
     wpe_view_backend_dispatch_axis_event (s_window->wpe.backend, &input_event.base);
-#else
-    assert (axis_delta[0] ^ axis_delta[1]);
-
-    struct wpe_input_axis_event input_event = {
-        .type = wpe_input_axis_event_type_motion,
-        .time = event->time,
-        .x = s_display->xcb.pointer.x,
-        .y = s_display->xcb.pointer.y,
-    };
-
-    if (!!axis_delta[0]) {
-        input_event.axis = 1;
-        input_event.value = axis_delta[0];
-    } else {
-        input_event.axis = 0;
-        input_event.value = axis_delta[1];
-    }
-
-    wpe_view_backend_dispatch_axis_event (s_window->wpe.backend, &input_event);
-#endif
 }
 
 static void
@@ -804,11 +799,8 @@ clear_glib (void)
     g_clear_pointer (&s_display->xcb.source, g_source_unref);
 }
 
-gboolean
-cog_platform_plugin_setup (CogPlatform *platform,
-                           CogShell    *shell G_GNUC_UNUSED,
-                           const char  *params,
-                           GError     **error)
+static gboolean
+cog_x11_platform_setup(CogPlatform *platform, CogShell *shell G_GNUC_UNUSED, const char *params, GError **error)
 {
     g_assert (platform);
     g_return_val_if_fail (COG_IS_SHELL (shell), FALSE);
@@ -870,11 +862,9 @@ cog_platform_plugin_setup (CogPlatform *platform,
     return TRUE;
 }
 
-void
-cog_platform_plugin_teardown (CogPlatform *platform)
+static void
+cog_x11_platform_finalize(GObject *object)
 {
-    g_assert (platform);
-
     clear_glib ();
     clear_gl ();
     clear_egl ();
@@ -883,12 +873,12 @@ cog_platform_plugin_teardown (CogPlatform *platform)
 
     g_clear_pointer (&s_window, free);
     g_clear_pointer (&s_display, free);
+
+    G_OBJECT_CLASS(cog_x11_platform_parent_class)->finalize(object);
 }
 
-WebKitWebViewBackend*
-cog_platform_plugin_get_view_backend (CogPlatform   *platform,
-                                      WebKitWebView *related_view,
-                                      GError       **error)
+static WebKitWebViewBackend *
+cog_x11_platform_get_view_backend(CogPlatform *platform, WebKitWebView *related_view, GError **error)
 {
     static struct wpe_view_backend_exportable_fdo_egl_client exportable_egl_client = {
         .export_fdo_egl_image = on_export_fdo_egl_image,
@@ -914,3 +904,60 @@ cog_platform_plugin_get_view_backend (CogPlatform   *platform,
 
     return wk_view_backend;
 }
+
+static void *
+check_supported(void *data G_GNUC_UNUSED)
+{
+    /*
+     * TODO: This could do more than only trying to connect to the X server.
+     */
+    Display *d = XOpenDisplay(NULL);
+    if (d) {
+        XCloseDisplay(d);
+        return GINT_TO_POINTER(TRUE);
+    } else {
+        return GINT_TO_POINTER(FALSE);
+    }
+}
+
+static gboolean
+cog_x11_platform_is_supported(void)
+{
+    static GOnce once = G_ONCE_INIT;
+    g_once(&once, check_supported, NULL);
+    return GPOINTER_TO_INT(once.retval);
+}
+
+static void
+cog_x11_platform_class_init(CogX11PlatformClass *klass)
+{
+    GObjectClass *object_class = G_OBJECT_CLASS(klass);
+    object_class->finalize = cog_x11_platform_finalize;
+
+    CogPlatformClass *platform_class = COG_PLATFORM_CLASS(klass);
+    platform_class->is_supported = cog_x11_platform_is_supported;
+    platform_class->setup = cog_x11_platform_setup;
+    platform_class->get_view_backend = cog_x11_platform_get_view_backend;
+}
+
+static void
+cog_x11_platform_class_finalize(CogX11PlatformClass *klass)
+{
+}
+
+static void
+cog_x11_platform_init(CogX11Platform *self)
+{
+}
+
+G_MODULE_EXPORT void
+g_io_cogplatform_x11_load(GIOModule *module)
+{
+    GTypeModule *type_module = G_TYPE_MODULE(module);
+    cog_x11_platform_register_type(type_module);
+}
+
+G_MODULE_EXPORT void
+g_io_cogplatform_x11_unload(GIOModule *module G_GNUC_UNUSED)
+{
+}
